//----------------------------------------------------------------------
//
//  FILE:   IceBenchmarkPlot.cpp
//  AUTHOR: Ian Ross
//  DATE:   28-MAR-2005
//
//----------------------------------------------------------------------
//
//  Class for MOTIF benchmark seasonal plot generation.
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <iostream>
#include <fstream>
#include <cstdio>
#include <netcdf.hh>

using namespace std;


// Local headers.

#include "IceBenchmarkPlot.hh"
#include "TempFile.hh"


//----------------------------------------------------------------------
//
//  MEMBER FUNCTION DEFINITIONS
//

IceBenchmarkPlot::IceBenchmarkPlot
(string model, const PlotSetup &setup, Season season, Hemisphere hemi) :
  BenchmarkPlot(model, ICE, "sic", "Sea Ice Concentration", SEASONAL, setup),
  _season(season), _hemi(hemi)
{
  char buff[256];
  string tmp_file = data_file();
  if (data_file() == "") {
    _plot_data_file = "";
    return;
  }

  int start, end;
  switch (_season) {
  case DJF: start = 11; end = 1; break;
  case JJA: start = 5;  end = 7; break;
  }

  sprintf(buff, "ncks -d time,%d,%d %s ###", start, end, data_file().c_str());
  tmp_file = TempFile::get("nc", buff);
  sprintf(buff, "ncra -d time,0,,1 %s ###", tmp_file.c_str());
  _plot_data_file = TempFile::get("nc", buff);
}


void IceBenchmarkPlot::plot(void)
{
  if (_plot_data_file == "") return;
  FILE *fp = popen("ncl", "w");
//   FILE *fp = fopen(TempFile::get("ncl").c_str(), "w");
  string outfile = TempFile::get("ps");
  string outfile2 = outfile.substr(0, outfile.size() - 3);

  fprintf(fp, "load \"$NCARG_ROOT/lib/ncarg/nclex/gsun/gsn_code.ncl\"\n");
  fprintf(fp, "load \"$NCARG_ROOT/lib/ncarg/nclex/gsun/gsn_csm.ncl\"\n");
  fprintf(fp, "begin\n");
  fprintf(fp, "fp = addfile(\"%s\", \"r\")\n", _plot_data_file.c_str());
  fprintf(fp, "var = fp->sic\n");
  fprintf(fp, "lat = fp->lat\n");
  fprintf(fp, "lon = fp->lon\n");
  fprintf(fp, "var@nlon = dimsizes(var&lon)\n");
  fprintf(fp, "var@nlat = dimsizes(var&lat)\n");
  fprintf(fp, "type = \"ps\"\n");
  fprintf(fp, "type@wkOrientation = \"landscape\"\n");
  fprintf(fp, "type@wkDeviceLowerY = 18 - 10\n");
  fprintf(fp, "type@wkDeviceUpperY = 792 - 18 - 10\n");
  fprintf(fp, "type@wkDeviceLowerX = 18 - 120\n");
  fprintf(fp, "type@wkDeviceUpperX = 792 - 18 - 120\n");
  fprintf(fp, "wks = gsn_open_wks(type, \"%s\")\n", outfile2.c_str());
  fprintf(fp, "res = True\n");
  fprintf(fp, "gsn_define_colormap(wks, \"rainbow+gray\")\n");
  if (_setup.single()) {
    fprintf(fp, "levels = (/ 1, 7, 13, 19, 25, 32, 44, 50, \\\n"
                "56, 62, 68, 74, 81, 87, 93, 99 /)\n");
    fprintf(fp, "fill_colours = ispan(2, 237, 13)\n");
    fprintf(fp, "fill_colours(0) = 0\n");
  } else {
    fprintf(fp, "levels = (/ -100, -80, -60, -40, -20, -10, -5, "
            "5, 10, 20, 40, 50, 60, 100 /)\n");
    fprintf(fp, "fill_colours = (/ 2, 31, 60, 88, 117, 146, 175, 0, "
            "176, 186, 196, 206, 217, 227, 237 /)\n");
  }
  fprintf(fp, "res@sfXCStartV = lon(0)\n");
  fprintf(fp, "res@sfXCEndV   = lon(0) + 360.0\n");
  fprintf(fp, "res@sfYCStartV = lat(0)\n");
  fprintf(fp, "res@sfYCEndV   = lat(var@nlat - 1)\n");
  fprintf(fp, "res@cnFillOn = True\n");
  fprintf(fp, "res@cnFillColors = fill_colours\n");
  fprintf(fp, "res@cnLevelSelectionMode = \"ExplicitLevels\"\n");
  fprintf(fp, "res@lbLabelFont = \"Helvetica\"\n");
  fprintf(fp, "res@cnLevels = levels\n");
  fprintf(fp, "res@cnMonoFillPattern = True\n");
  fprintf(fp, "res@cnMonoFillColor = False\n");
  fprintf(fp, "res@cnLineLabelsOn = False\n");
  fprintf(fp, "res@cnInfoLabelOn = False\n");
  fprintf(fp, "res@cnLinesOn = False\n");
  fprintf(fp, "res@sfYArray = lat\n");
  fprintf(fp, "res@vpYF = 0.85\n");
  fprintf(fp, "res@vpXF = 0.05\n");
  fprintf(fp, "res@vpWidthF = 0.9 * 0.50\n");
  fprintf(fp, "res@vpHeightF = 0.9 * 0.55\n");
  fprintf(fp, "res@sfMissingValueV = 1.0E20\n");
  fprintf(fp, "res@mpGridAndLimbOn = True\n");
  fprintf(fp, "res@mpGridLineColor = 238\n");
  if (_hemi == NORTH) {
    fprintf(fp, "res@gsnPolar = \"NH\"\n");
    fprintf(fp, "res@mpMinLatF = 40.0\n");
  } else {
    fprintf(fp, "res@gsnPolar = \"SH\"\n");
    fprintf(fp, "res@mpMaxLatF = -40.0\n");
  }
  fprintf(fp, "res@mpPerimOn = True\n");
  fprintf(fp, "res@mpShapeMode = \"FreeAspect\"\n");
  fprintf(fp, "res@mpPerimDrawOrder = \"PostDraw\"\n");
  fprintf(fp, "res@tmXBLabelFont = \"Helvetica\"\n");
  fprintf(fp, "res@tmYLLabelFont = \"Helvetica\"\n");
  fprintf(fp, "res@pmLabelBarDisplayMode = \"Never\"\n");
  fprintf(fp, "res@lbLabelBarOn = True\n");
  fprintf(fp, "res@lbTitleString = \"Sea Ice Fraction (%%)\"\n");
  fprintf(fp, "res@lbTitleFont = \"Helvetica\"\n");
  fprintf(fp, "res@lbTitleFontHeightF = 0.01\n");
  fprintf(fp, "res@lbTitlePosition = \"Bottom\"\n");
  fprintf(fp, "res@lbTitleOn = True\n");
  fprintf(fp, "res@lbLabelAngleF = 90.0\n");
  if (!_setup.single()) {
    fprintf(fp, "res@lbLabelStrings = (/ \"-100\", \"-80\", \"-60\", "
            "\"-40\", \"-20\", \"-10\", \"-5\", "
            "\"+5\", \"+10\", \"+20\", \"+40\", \"+50\", \"+60\", "
            "\"+100\" /)\n");
  }
  fprintf(fp, "res@tmXBLabelFontHeightF = 0.01\n");
  fprintf(fp, "res@tmXBMajorLengthF = 0.005\n");
  fprintf(fp, "res@mpGridAndLimbOn = True\n");
  fprintf(fp, "res@mpGridLonSpacingF = 30\n");
  fprintf(fp, "res@mpGridSpacingF = 10\n");
  fprintf(fp, "res@gsnStringFont = \"Helvetica\"\n");
  fprintf(fp, "res@gsnLeftString = \"%s\"\n", title_string().c_str());
  fprintf(fp, "res@gsnRightString = \"%s\"\n", date_mark().c_str());
  fprintf(fp, "plot = gsn_csm_contour_map_polar(wks, var(0,:,:), res)\n");
  fprintf(fp, "end\n");
  pclose(fp);

  string command = string("ps2eps -q ") + outfile;
  system(command.c_str());
  rename((outfile2 + ".eps").c_str(), eps_file().c_str());
  set_valid();
}


string IceBenchmarkPlot::title_string(void)
{
  char buff[256];
  string season_string, hemi_string;
  switch(_season) {
  case DJF: season_string = "DJF"; break;
  case JJA: season_string = "JJA"; break;
  }
  switch(_hemi) {
  case NORTH: hemi_string = "NH"; break;
  case SOUTH: hemi_string = "SH"; break;
  }

  sprintf(buff, "%s %s %s Sea Ice (%s)",
          _model.c_str(), season_string.c_str(), hemi_string.c_str(),
          string(_setup).c_str());
   
  return buff;
}


//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
