//----------------------------------------------------------------------
//
//  FILE:   PlotCollator.hh
//  AUTHOR: Ian Ross
//  DATE:   29-MAR-2005
//
//----------------------------------------------------------------------
//
//  Plot collation class for MOTIF benchmarking plot document generator.
//
//----------------------------------------------------------------------

#ifndef _H_PLOT_COLLATOR_
#define _H_PLOT_COLLATOR_

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <string>

using namespace std;


// Local headers.

#include "PlotSetup.hh"
#include "BenchmarkPlot.hh"


//----------------------------------------------------------------------
//
//  CLASS DEFINITION
//

class PlotCollator {
public:

  PlotCollator(string outfile, string model, string title);
  ~PlotCollator();

  void new_variable(string var_name);
  void new_setup(const PlotSetup &setup);
  void add_plot(const BenchmarkPlot *plot, float angle = 0.0);

  void operator()(void);


private:

  string _outfile;
  string _model;
  string _title;

  struct PlotEntry {
    const BenchmarkPlot *plot;
    float angle;

    PlotEntry(const BenchmarkPlot *pl, float an) :
      plot(pl), angle(an) { }
  };

  struct SetupEntry {
    SetupEntry(const PlotSetup &s) : setup(s) { }
    PlotSetup setup;
    vector<PlotEntry *> plots;
  };

  struct VariableEntry {
    VariableEntry(string v) : var_name(v) { }
    string var_name;
    vector<SetupEntry *> setups;
  };

  vector<VariableEntry *> _vars;

  VariableEntry *_curr_var;
  SetupEntry *_curr_setup;
};

#endif

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
