//----------------------------------------------------------------------
//
//  FILE:   PlotSetup.cpp
//  AUTHOR: Ian Ross
//  DATE:   28-MAR-2005
//
//----------------------------------------------------------------------
//
//  Basic plot setup definitions (i.e. which run or pair or runs for
//  anomaly plots) for MOTIF benchmarking tools.
//
//----------------------------------------------------------------------

// HEADER FILES

#include "PlotSetup.hh"


//----------------------------------------------------------------------
//
//  STATIC MEMBER INITIALISERS
//

bool PlotSetup::_vis_strings = true;


//----------------------------------------------------------------------
//
//  MEMBER FUNCTION DEFINITIONS
//

ModelRun PlotSetup::run(void) const
{
  if (!_single) throw "INTERNAL: PlotSetup - not a single element setup";
  return _run1;
}

ModelRun PlotSetup::experiment(void) const
{
  if (_single) throw "INTERNAL: PlotSetup - not an anomaly setup";
  return _run1;
}

ModelRun PlotSetup::control(void) const
{
  if (_single) throw "INTERNAL: PlotSetup - not an anomaly setup";
  return _run2;
}

static string run_to_string(ModelRun run)
{
  if (PlotSetup::_vis_strings) {
    switch (run) {
    case OA_0K:    return "0K OA";
    case OAV_0K:   return "0K OAV";
    case OA_6K:    return "6K OA";
    case OAV_6K:   return "6K OAV";
    case OA_21K:   return "21K OA";
    case OAV_21K:  return "21K OAV";
    }
  } else {
    switch (run) {
    case OA_0K:    return "0K_OA";
    case OAV_0K:   return "0K_OAV";
    case OA_6K:    return "6K_OA";
    case OAV_6K:   return "6K_OAV";
    case OA_21K:   return "21K_OA";
    case OAV_21K:  return "21K_OAV";
    }
  }
}

PlotSetup::operator string()
{
  if (single())
    return run_to_string(run());
  else {
    if (_vis_strings)
      return run_to_string(experiment()) + " - " + run_to_string(control());
    else
      return run_to_string(experiment()) + "-" + run_to_string(control());
  }
}

ostream &operator<<(ostream &str, const PlotSetup &setup)
{
  if (setup.single())
    str << run_to_string(setup.run());
  else
    str << run_to_string(setup.experiment()) << " - "
        << run_to_string(setup.control());
  return str;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
