//----------------------------------------------------------------------
//
//  FILE:   SeasonalBenchmarkPlot.cpp
//  AUTHOR: Ian Ross
//  DATE:   28-MAR-2005
//
//----------------------------------------------------------------------
//
//  Class for MOTIF benchmark seasonal plot generation.
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <cstdio>

using namespace std;


// Local headers.

#include "SeasonalBenchmarkPlot.hh"
#include "TempFile.hh"


//----------------------------------------------------------------------
//
//  MEMBER FUNCTION DEFINITIONS
//

SeasonalBenchmarkPlot::SeasonalBenchmarkPlot
(string model, Domain domain, string var_name, string var_full_name,
 const PlotSetup &setup, Season season) :
  BenchmarkPlot(model, domain, var_name, var_full_name, SEASONAL, setup),
  _season(season)
{
  char buff[256];
  string tmp_file = data_file();
  if (data_file() == "") {
    _seasonal_data_file = "";
    return;
  }

  int start, end;
  switch (_season) {
  case ANNUAL: start = 0;  end = 11;  break;
  case DJF: start = 11; end = 1; break;
  case MAM: start = 2;  end = 4; break;
  case JJA: start = 5;  end = 7; break;
  case SON: start = 8;  end = 10; break;
  case JAS: start = 6;  end = 8; break;
  }

  if (_var_name == "wap") {
    sprintf(buff, "ncks -d time,%d,%d -d plev,5 %s ###",
            start, end, data_file().c_str());
    string tmp_file1 = TempFile::get("nc", buff);
    sprintf(buff, "ncwa -a plev %s ###", tmp_file1.c_str());
    tmp_file = TempFile::get("nc", buff);
  } else {
    sprintf(buff, "ncks -d time,%d,%d %s ###",
            start, end, data_file().c_str());
    tmp_file = TempFile::get("nc", buff);
  }

  sprintf(buff, "ncra -d time,0,,1 %s ###", tmp_file.c_str());
  _seasonal_data_file = TempFile::get("nc", buff);
}


void SeasonalBenchmarkPlot::plot(void)
{
  if (_seasonal_data_file == "") return;
  map_plot(_seasonal_data_file);
  set_valid();
}


void SeasonalBenchmarkPlot::colours(FILE *fp, int &grid_colour)
{
  string cmap;
  if (_var_name == "tas" || _var_name == "tos" ||
      _var_name == "pr" || _var_name == "mrso") {
    cmap = "BlueRedGray"; grid_colour = 254;
  } else {
    cmap = "rainbow+gray"; grid_colour = 238;
  }
  fprintf(fp, "gsn_define_colormap(wks, \"%s\")\n", cmap.c_str());

  float lmin, lmax;
  int lcnt, cmin, cmax, cstr;
  if (_var_name == "tas") {
    if (_setup.single()) {
      fprintf(fp, "levels = fspan(-80, 50, 14)\n");
      fprintf(fp, "fill_colours = "
              "(/ 2, 17, 32, 47, 62, 77, 92, 107, 127, "
              "128, 153, 178, 203, 228, 253 /)\n");
    } else {
      fprintf(fp, "levels = (/ -20,-15,-10,-5,-2,-1,-0.1,"
              "0.1,1,2,5,10,15,20 /)\n");
      fprintf(fp, "fill_colours = "
              "(/ 2, 23, 44, 64, 85, 106, 127, 0, "
              "128, 149, 170, 190, 211, 232, 253 /)\n");
    }
  } else if (_var_name == "tos") {
    if (_setup.single()) {
      fprintf(fp, "levels = (/ -2,-1,0,2,5,10,15,20,25,30,35,40,45,50 /)\n");
      fprintf(fp, "fill_colours = "
              "(/ 2, 17, 32, 128, 139, 151, 162, 173, 185, "
              "196, 208, 219, 230, 242, 253 /)\n");
    } else {
      fprintf(fp, "levels = (/ -20,-15,-10,-5,-2,-1,-0.1,"
              "0.1,1,2,5,10,15,20 /)\n");
      fprintf(fp, "fill_colours = "
              "(/ 2, 23, 44, 64, 85, 106, 127, 0, "
              "128, 149, 170, 190, 211, 232, 253 /)\n");
    }
  } else if (_var_name == "mrso") {
    if (_setup.single()) {
      fprintf(fp, "levels = (/ 100, 250, 500, 1000, 1500, 2000 /)\n");
      fprintf(fp, "fill_colours = (/ 0, 79, 64, 48, 33, 17, 2 /)\n");
    } else {
      fprintf(fp, "levels = "
              "(/ -1000,-750,-600,-500,-400,-350,-300,-250,-200,-150,"
              "-100,-50,-25,-1,"
              "1,25,50,100,150,200,250,300,350,400,500,600,750,1000 /)\n");
      fprintf(fp, "fill_colours = "
              "(/ 253,243,234,224,215,205,195,186,176,166,157,147,138,128,0,"
              "127,117,108,98,89,79,69,60,50,40,31,21,12,2 /)\n");
      fprintf(fp, "res@lbLabelAngleF = 90.0\n");
    }
  } else if (_var_name == "pr") {
    if (_setup.single()) {
      fprintf(fp, "levels = (/ 2, 4, 8, 12, 16, 30 /)\n");
      fprintf(fp, "fill_colours = (/ 0, 79, 64, 48, 33, 17, 2 /)\n");
    } else {
      fprintf(fp, "levels = "
              "(/ -20,-18,-16,-14,-12,-10,-8,-6,-4,-2,-1,-0.5,-0.25,-0.01,"
              "0.01,0.25,0.5,1,2,4,6,8,10,12,14,16,18,20 /)\n");
      fprintf(fp, "fill_colours = "
              "(/ 253,243,234,224,215,205,195,186,176,166,157,147,138,128,0,"
              "127,117,108,98,89,79,69,60,50,40,31,21,12,2 /)\n");
      fprintf(fp, "res@lbLabelAngleF = 90.0\n");
    }
  } else if (_var_name == "psl") {
      if (_setup.single()) {
        fprintf(fp, "levels = fspan(960, 1040, 17)\n");
        fprintf(fp, "fill_colours = ispan(2, 237, 13)\n");
      } else {
        fprintf(fp, "levels = (/ -40, -20, -15, -10, -5, -2, -1, "
                "1, 2, 5, 10, 15, 20, 40 /)\n");
        fprintf(fp, "fill_colours = (/ 2, 31, 60, 88, 117, 146, 175, 0, "
                "176, 186, 196, 206, 217, 227, 237 /)\n");
      }
  } else {
    if (_var_name == "uas" || _var_name == "vas") {
      fprintf(fp, "levels = (/ -10, -8, -6, -4, -2, -0.5, "
              "0.5, 2, 4, 6, 8, 10 /)\n");
      fprintf(fp, "fill_colours = (/ 2, 27, 52, 77, 102, 127, 0, "
              "175, 187, 200, 212, 225, 237/)\n");
    } else if (_var_name == "wap") {
      fprintf(fp, "levels = (/ -0.25, -0.2, -0.15, -0.1, -0.05, -0.005, "
              "0.005, 0.05, 0.1, 0.15, 0.2, 0.25 /)\n");
      fprintf(fp, "fill_colours = (/ 2, 27, 52, 77, 102, 127, 0, "
              "175, 187, 200, 212, 225, 237/)\n");
    }
  }
}


void SeasonalBenchmarkPlot::scaling
(float &scale_factor, float &offset, string &units)
{
  scale_factor = 1.0;
  offset = 0.0;
  if (_var_name == "tas" || _var_name == "tos") {
    // Input data is in Kelvin; plots should be in deg. C.
    if (_setup.single()) offset = -273.16;
    units = ":S:o:N:C";
  } else if (_var_name == "pr") {
    // Input data is in kg m-2 s-1; plots should be in mm day-1.
    scale_factor = 3600 * 24;
    units = "mm/day";
  } else if (_var_name == "psl") {
    // Input data is in Pa; plots should be in hPa.
    scale_factor = 0.01;
    units = "hPa";
  } else if (_var_name == "uas" || _var_name == "vas") {
    // Input data is already in the appropriate units.
    units = "m/s";
  } else if (_var_name == "wap") {
    // Input data is already in the appropriate units.
    units = "Pa/s";
  } else if (_var_name == "mrso") {
    units = "kg/m:S:2:N:";
  }
}


string SeasonalBenchmarkPlot::title_string(void)
{
  string season_string;
  switch(_season) {
  case ANNUAL: season_string = "Annual"; break;
  case DJF: season_string = "DJF"; break;
  case MAM: season_string = "MAM"; break;
  case JJA: season_string = "JJA"; break;
  case SON: season_string = "SON"; break;
  case JAS: season_string = "JAS"; break;
  }

  char buff[256];
  sprintf(buff, "%s %s %s (%s)",
          _model.c_str(), season_string.c_str(), _var_full_name.c_str(),
          string(_setup).c_str());
  return buff;
}


//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
