//----------------------------------------------------------------------
//
//  FILE:   motif-zonal-precip.cpp
//  AUTHOR: Ian Ross
//  DATE:   28-MAR-2005
//
//----------------------------------------------------------------------
//
//  Main program for MOTIF benchmarking plot document generator.
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
//  HEADER FILES
//

// Standard headers.

#include <iostream>
#include <cstdlib>
#include <string>

using namespace std;


// Local headers.

#include "PlotSetup.hh"
#include "ZonalPrecipData.hh"
#include "TempFile.hh"


//----------------------------------------------------------------------
//
//  MAIN PROGRAM
//

int main(int argc, char *argv[])
{
  PlotSetup::_vis_strings = false;

  // There should just be one command line argument - the model to
  // deal with.

  if (argc != 2) {
    cerr << "Usage: motif-zonal-precip <model>" << endl;
    exit(1);
  }
  string model = argv[1];

  try {
    // Set up plot configuration list.

    vector<PlotSetup> setups_6k;
    setups_6k.push_back(PlotSetup(OA_6K));
    setups_6k.push_back(PlotSetup(OAV_6K));
    setups_6k.push_back(PlotSetup(OA_6K, OA_0K));
    setups_6k.push_back(PlotSetup(OAV_6K, OAV_0K));

    for (vector<PlotSetup>::iterator sit = setups_6k.begin();
         sit != setups_6k.end(); ++sit) {
      ZonalPrecipData *zonal_africa = new ZonalPrecipData(model, *sit);
      zonal_africa->plot();
    }

    TempFile::clean();
  } catch (const char *msg) {
    cout << "EXCEPTION: " << msg << endl;
  }
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------

